@@ -1,5 +1,5 @@
 /* merge.c - Functions which actually combine and manipulate GIF image data.
-   Copyright (C) 1997-2013 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2014 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -43,10 +43,14 @@ mark_used_colors(Gif_Stream *gfs, Gif_Image *gfi, Gt_Crop *crop,
                  int compress_immediately)
 {
     Gif_Colormap *gfcm = gfi->local ? gfi->local : gfs->global;
-    Gif_Color *col = gfcm->col;
-    int ncol = gfcm->ncol;
-    int transp = gfi->transparent;
-    int i, j, l, t, r, b, nleft;
+    Gif_Color *col;
+    int i, j, l, t, r, b, nleft, ncol, transp = gfi->transparent;
+
+    /* There might not be a colormap. */
+    if (!gfcm)
+        return;
+    col = gfcm->col;
+    ncol = gfcm->ncol;
 
     /* Mark color used for transparency. */
     if (transp >= 0 && transp < ncol)
@@ -60,7 +64,7 @@ mark_used_colors(Gif_Stream *gfs, Gif_Image *gfi, Gt_Crop *crop,
     if (nleft == 0)
         return;
 
-    if (gfi->img || Gif_UncompressImage(gfi) == 2)
+    if (gfi->img || Gif_UncompressImage(gfs, gfi) == 2)
         compress_immediately = 0;
 
     /* Loop over every pixel (until we've seen all colors) */
@@ -108,77 +112,83 @@ find_color_index(Gif_Color *c, int nc, Gif_Color *color)
 int
 merge_colormap_if_possible(Gif_Colormap *dest, Gif_Colormap *src)
 {
-  Gif_Color *srccol = src->col;
-  Gif_Color *destcol = dest->col;
-  int ndestcol = dest->ncol;
-  int dest_userflags = dest->userflags;
-  int i, x;
-  int trivial_map = 1;
+    Gif_Color *srccol;
+    Gif_Color *destcol = dest->col;
+    int ndestcol = dest->ncol;
+    int dest_userflags = dest->userflags;
+    int i, x;
+    int trivial_map = 1;
 
-  for (i = 0; i < src->ncol; i++) {
-    if (srccol[i].haspixel & 1) {
-      /* Store an image color cell's mapping to the global colormap in its
-	 'pixel' slot. This is useful caching: oftentimes many input frames
-	 will share a colormap */
-      int mapto = (srccol[i].pixel < 256 ? (int)srccol[i].pixel : -1);
+    if (!src)
+        return 1;
 
-      if (mapto == -1)
-	mapto = find_color_index(destcol, ndestcol, &srccol[i]);
+    srccol = src->col;
+    for (i = 0; i < src->ncol; i++) {
+        if (srccol[i].haspixel & 1) {
+            /* Store an image color cell's mapping to the global colormap in
+               its 'pixel' slot. This is useful caching: oftentimes many
+               input frames will share a colormap */
+            int mapto = (srccol[i].pixel < 256 ? (int)srccol[i].pixel : -1);
 
-      if (mapto == -1 && ndestcol < 256) {
-	/* add the color */
-	mapto = ndestcol;
-	destcol[mapto] = srccol[i];
-	ndestcol++;
-      }
+            if (mapto == -1)
+                mapto = find_color_index(destcol, ndestcol, &srccol[i]);
 
-      if (mapto == -1)
-	/* check for a pure-transparent color */
-	for (x = 0; x < ndestcol; x++)
-	  if (destcol[x].haspixel == 2) {
-	    mapto = x;
-	    destcol[mapto] = srccol[i];
-	    break;
-	  }
+            if (mapto == -1 && ndestcol < 256) {
+                /* add the color */
+                mapto = ndestcol;
+                destcol[mapto] = srccol[i];
+                ndestcol++;
+            }
 
-      if (mapto == -1)
-	/* give up and require a local colormap */
-	goto local_colormap_required;
+            if (mapto == -1)
+                /* check for a pure-transparent color */
+                for (x = 0; x < ndestcol; x++)
+                    if (destcol[x].haspixel == 2) {
+                        mapto = x;
+                        destcol[mapto] = srccol[i];
+                        break;
+                    }
 
-      assert(mapto >= 0 && mapto < ndestcol);
-      assert(GIF_COLOREQ(&destcol[mapto], &srccol[i]));
+            if (mapto == -1)
+                /* give up and require a local colormap */
+                goto local_colormap_required;
 
-      srccol[i].pixel = mapto;
-      destcol[mapto].haspixel = 1;
-      if (mapto != i)
-	trivial_map = 0;
+            assert(mapto >= 0 && mapto < ndestcol);
+            assert(GIF_COLOREQ(&destcol[mapto], &srccol[i]));
 
-    } else if (srccol[i].haspixel & 2) {
-      /* a dedicated transparent color; if trivial_map & at end of colormap
-         insert it with haspixel == 2. (strictly not necessary; we do it to
-	 try to keep the map trivial.) */
-      if (trivial_map && i == ndestcol) {
-	destcol[ndestcol] = srccol[i];
-	ndestcol++;
-      }
+            srccol[i].pixel = mapto;
+            destcol[mapto].haspixel = 1;
+            if (mapto != i)
+                trivial_map = 0;
+
+        } else if (srccol[i].haspixel & 2) {
+            /* a dedicated transparent color; if trivial_map & at end of
+               colormap insert it with haspixel == 2. (strictly not
+               necessary; we do it to try to keep the map trivial.) */
+            if (trivial_map && i == ndestcol) {
+                destcol[ndestcol] = srccol[i];
+                ndestcol++;
+            }
+        }
     }
-  }
 
-  /* success! save new number of colors */
-  dest->ncol = ndestcol;
-  dest->userflags = dest_userflags;
-  return 1;
+    /* success! save new number of colors */
+    dest->ncol = ndestcol;
+    dest->userflags = dest_userflags;
+    return 1;
 
   /* failure: a local colormap is required */
  local_colormap_required:
-  if (warn_local_colormaps == 1) {
-    static int context = 0;
-    warning(1, "so many colors that local colormaps were required");
-    if (!context)
-      warncontext(1, "(You may want to try '--colors 256'.)");
-    warn_local_colormaps = 2;
-    context = 1;
-  }
+    if (warn_local_colormaps == 1) {
+        static int context = 0;
+        if (!context) {
+            warning(1, "too many colors, using local colormaps\n"
+                    "  (You may want to try %<--colors 256%>.)");
+            context = 1;
+        } else
+            warning(1, "too many colors, using local colormaps");
+        warn_local_colormaps = 2;
+    }
 
   /* 9.Dec.1998 - This must have been a longstanding bug! We MUST clear
      the cached mappings of any pixels in the source colormap we
@@ -226,81 +236,82 @@ merge_comments(Gif_Comment *destc, Gif_Comment *srcc)
 }
 
 
+static void merge_image_input_colors(uint8_t* inused, const Gif_Image* srci) {
+    int i, x, y, nleft = Gif_ImageColorBound(srci);
+    for (i = 0; i != 256; ++i)
+        inused[i] = 0;
+    for (y = 0; y != srci->height && nleft > 0; ++y) {
+        const uint8_t* data = srci->img[y];
+        for (x = 0; x != srci->width; ++x, ++data) {
+            nleft -= 1 - inused[*data];
+            inused[*data] = 1;
+        }
+    }
+    if (srci->transparent >= 0)
+        inused[srci->transparent] = 0;
+}
+
+
 Gif_Image *
 merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci,
-	    int same_compressed_ok)
+	    Gt_Frame* srcfr, int same_compressed_ok)
 {
   Gif_Colormap *imagecm;
-  Gif_Color *imagecol;
-  int islocal;
   int i;
   Gif_Colormap *localcm = 0;
   Gif_Colormap *destcm = dest->global;
 
   uint8_t map[256];		/* map[input pixel value] == output pixval */
-  int trivial_map = 1;		/* does the map take input pixval --> the same
+  int trivial_map;		/* does the map take input pixval --> the same
 				   pixel value for all colors in the image? */
+  uint8_t inused[256];          /* inused[input pival] == 1 iff used */
   uint8_t used[256];		/* used[output pixval K] == 1 iff K was used
 				   in the image */
 
   Gif_Image *desti;
 
   /* mark colors that were actually used in this image */
-  islocal = srci->local != 0;
-  imagecm = islocal ? srci->local : src->global;
-  if (!imagecm)
-    fatal_error("no global or local colormap for source image");
-  imagecol = imagecm->col;
-  {
-      int ncol = imagecm->ncol, nleft = 256 - ncol;
-      int w = srci->width, h = srci->height, i, j;
-      for (i = 0; i != ncol; ++i)
-          imagecol[i].haspixel &= ~4;
-      for (j = 0; j != h; ++j) {
-          uint8_t *data = srci->img[j];
-          for (i = 0; i != w; ++i, ++data)
-              if (*data < ncol && !(imagecol[*data].haspixel & 4)) {
-                  imagecol[*data].haspixel |= 4;
-                  --nleft;
-                  if (nleft == 0)
-                      goto done_mark;
-              }
+  imagecm = srci->local ? srci->local : src->global;
+  merge_image_input_colors(inused, srci);
+  for (i = imagecm ? imagecm->ncol : 0; i != 256; ++i)
+      if (inused[i]) {
+          lwarning(srcfr->input_filename, "some colors undefined by colormap");
+          break;
       }
-  done_mark:
-      if (srci->transparent >= 0 && srci->transparent < ncol)
-          imagecol[srci->transparent].haspixel &= ~4;
-  }
 
   /* map[old_pixel_value] == new_pixel_value */
   for (i = 0; i < 256; i++)
-    map[i] = used[i] = 0;
+      map[i] = used[i] = 0;
 
   /* Merge the colormap */
   if (merge_colormap_if_possible(dest->global, imagecm)) {
-    /* Create 'map' and 'used' for global colormap. */
-    for (i = 0; i < imagecm->ncol; i++)
-      if (imagecol[i].haspixel & 4) {
-	map[i] = imagecol[i].pixel;
-	if (map[i] != i)
-            trivial_map = 0;
-	used[map[i]] = 1;
-      }
+      /* Create 'map' and 'used' for global colormap. */
+      for (i = 0; i != 256; ++i)
+          if (inused[i]) {
+              if (imagecm && i < imagecm->ncol)
+                  map[i] = imagecm->col[i].pixel;
+              else
+                  map[i] = 0;
+          }
 
   } else {
-    /* Need a local colormap. */
-    int ncol = 0;
-    destcm = localcm = Gif_NewFullColormap(0, 256);
-    for (i = 0; i < imagecm->ncol; i++)
-      if (imagecol[i].haspixel & 4) {
-	map[i] = ncol;
-	if (ncol != i)
-            trivial_map = 0;
-        used[ncol] = 1;
-	localcm->col[ ncol++ ] = imagecol[i];
-      }
-    localcm->ncol = ncol;
+      /* Need a local colormap. */
+      destcm = localcm = Gif_NewFullColormap(0, 256);
+      for (i = 0; i != 256; ++i)
+          if (inused[i]) {
+              map[i] = localcm->ncol;
+              localcm->col[localcm->ncol] = imagecm->col[i];
+              ++localcm->ncol;
+          }
   }
 
+  trivial_map = 1;
+  for (i = 0; i != 256; ++i)
+      if (inused[i]) {
+          used[map[i]] = 1;
+          trivial_map = trivial_map && map[i] == i;
+      }
+
   /* Decide on a transparent index */
   if (srci->transparent >= 0) {
     int found_transparent = -1;
@@ -321,8 +332,8 @@ merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci,
       /* 1.Aug.1999 - Don't update destcm->ncol -- we want the output colormap
          to be as small as possible. */
       c = &destcm->col[found_transparent];
-      if (srci->transparent < imagecm->ncol)
-	*c = imagecol[srci->transparent];
+      if (imagecm && srci->transparent < imagecm->ncol)
+	*c = imagecm->col[srci->transparent];
       c->haspixel = 2;
       assert(c->haspixel == 2 && found_transparent < 256);
     }
@@ -356,11 +367,11 @@ merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci,
   if (trivial_map && same_compressed_ok && srci->compressed) {
     desti->compressed_len = srci->compressed_len;
     desti->compressed = Gif_NewArray(uint8_t, srci->compressed_len);
-    desti->free_compressed = Gif_DeleteArrayFunc;
+    desti->free_compressed = Gif_Free;
     memcpy(desti->compressed, srci->compressed, srci->compressed_len);
   } else {
     int i, j;
-    Gif_CreateUncompressedImage(desti);
+    Gif_CreateUncompressedImage(desti, desti->interlace);
 
     if (trivial_map)
       for (j = 0; j < desti->height; j++)
